@@ -35,6 +35,9 @@ use crate::{
     circuit_node_private::{
         CircuitNodeComputeInfoImpl, CircuitNodeHashItems, CircuitNodeHashWithChildren,
     },
+    circuit_utils::{
+        child_name_with_maybe_paren, children_names_with_maybe_paren, OperatorPriority,
+    },
     new_rc_unwrap,
     prelude::*,
     CachedCircuitInfo, CircuitNodeSelfOnlyHash, ConstructError, PyCircuitBase, Scalar, Shape,
@@ -245,18 +248,19 @@ impl Einsum {
 }
 
 impl CircuitNodeAutoName for Einsum {
+    const PRIORITY: OperatorPriority = OperatorPriority::Infix { priority: 1 };
+
     fn auto_name(&self, name: Option<String>) -> Option<String> {
         name.or_else(|| {
-            if self.input_circuits().any(|x| x.name().is_none()) {
-                None
-            } else {
-                Some(
-                    self.input_circuits()
-                        .map(|x| Self::shorten_child_name(x.name().unwrap()))
+            children_names_with_maybe_paren(&Self::PRIORITY, self.children().collect()).map(
+                |names| {
+                    names
+                        .iter()
+                        .map(|name| Self::shorten_child_name(name))
                         .collect::<Vec<String>>()
-                        .join(" * "),
-                )
-            }
+                        .join(" * ")
+                },
+            )
         })
     }
 }
@@ -699,18 +703,19 @@ impl CircuitNode for Add {
 }
 
 impl CircuitNodeAutoName for Add {
+    const PRIORITY: OperatorPriority = OperatorPriority::Infix { priority: 0 };
+
     fn auto_name(&self, name: Option<String>) -> Option<String> {
         name.or_else(|| {
-            if self.children().any(|x| x.name().is_none()) {
-                None
-            } else {
-                Some(
-                    self.children()
-                        .map(|x| Self::shorten_child_name(x.name().unwrap()))
+            children_names_with_maybe_paren(&Self::PRIORITY, self.children().collect()).map(
+                |names| {
+                    names
+                        .iter()
+                        .map(|name| Self::shorten_child_name(name))
                         .collect::<Vec<String>>()
-                        .join(" + "),
-                )
-            }
+                        .join(" + ")
+                },
+            )
         })
     }
 }
@@ -894,8 +899,13 @@ impl CircuitNode for Rearrange {
 }
 
 impl CircuitNodeAutoName for Rearrange {
+    const PRIORITY: OperatorPriority = OperatorPriority::PostFix {};
+
     fn auto_name(&self, name: Option<String>) -> Option<String> {
-        name.or_else(|| self.node.name().map(|n| "rearrange ".to_owned() + n))
+        name.or_else(|| {
+            child_name_with_maybe_paren(&Self::PRIORITY, self.node.clone())
+                .map(|n| n + " rearrange")
+        })
     }
 }
 
@@ -1108,8 +1118,12 @@ impl CircuitNode for Index {
 }
 
 impl CircuitNodeAutoName for Index {
+    const PRIORITY: OperatorPriority = OperatorPriority::PostFix {};
+
     fn auto_name(&self, name: Option<String>) -> Option<String> {
-        name.or_else(|| self.node.name().map(|n| "idx ".to_owned() + n))
+        name.or_else(|| {
+            child_name_with_maybe_paren(&Self::PRIORITY, self.node.clone()).map(|n| n + " idx")
+        })
     }
 }
 
@@ -1394,18 +1408,19 @@ impl CircuitNode for Concat {
 }
 
 impl CircuitNodeAutoName for Concat {
+    const PRIORITY: OperatorPriority = OperatorPriority::InfixAmbiguous {};
+
     fn auto_name(&self, name: Option<String>) -> Option<String> {
         name.or_else(|| {
-            if self.children().any(|x| x.name().is_none()) {
-                None
-            } else {
-                Some(
-                    self.children()
-                        .map(|x| Self::shorten_child_name(x.name().unwrap()))
+            children_names_with_maybe_paren(&Self::PRIORITY, self.children().collect()).map(
+                |names| {
+                    names
+                        .iter()
+                        .map(|name| Self::shorten_child_name(name))
                         .collect::<Vec<String>>()
-                        .join(" ++ "),
-                )
-            }
+                        .join(" ++ ")
+                },
+            )
         })
     }
 }
@@ -1584,8 +1599,12 @@ impl CircuitNode for Scatter {
 }
 
 impl CircuitNodeAutoName for Scatter {
+    const PRIORITY: OperatorPriority = OperatorPriority::PostFix {};
+
     fn auto_name(&self, name: Option<String>) -> Option<String> {
-        name.or_else(|| self.node.name().map(|n| "scatter ".to_owned() + n))
+        name.or_else(|| {
+            child_name_with_maybe_paren(&Self::PRIORITY, self.node.clone()).map(|n| n + " scatter")
+        })
     }
 }
 
@@ -1860,8 +1879,14 @@ impl CircuitNode for Conv {
 }
 
 impl CircuitNodeAutoName for Conv {
+    const PRIORITY: OperatorPriority = OperatorPriority::Function {};
+
     fn auto_name(&self, name: Option<String>) -> Option<String> {
-        name.or_else(|| self.input.name().map(|n| "Conv ".to_owned() + n))
+        name.or_else(|| {
+            let input_name = self.input.name()?;
+            let filter_name = self.filter.name()?;
+            Some(format!("Conv({input_name}, {filter_name})"))
+        })
     }
 }
 
