@@ -367,9 +367,10 @@ pub fn expand_node(circuit: CircuitRc, inputs: &Vec<CircuitRc>) -> Result<Circui
                 .into_iter()
                 .unzip();
 
-            let spec = ModuleSpec::new(spec_circuit, arg_specs, false)
-                .context("constructing new ModuleSpec failed in expand")?;
-
+            let spec = ModuleSpec {
+                circuit: spec_circuit,
+                arg_specs,
+            };
             Module::try_new(nodes, spec, node.name_cloned()).map(|z| z.rc())
         }
         Circuit::Tag(node) => Ok(Tag::new(inputs[0].clone(), node.uuid, node.name_cloned()).rc()),
@@ -390,16 +391,16 @@ pub fn expand_node(circuit: CircuitRc, inputs: &Vec<CircuitRc>) -> Result<Circui
 #[base_exception(PyValueError)]
 #[derive(Error, Debug, Clone)]
 pub enum ExpandError {
-    #[error("expand wrong number of children, expected {expected} got {got}")]
+    #[error("expand wrong number of children, expected {expected} got {got} ({e_name})")]
     WrongNumChildren { expected: usize, got: usize },
 
-    #[error("Batching Rank Too Low")]
+    #[error("Batching Rank Too Low ({e_name})")]
     BatchingRankTooLow {
         default: Vec<usize>,
         got: Vec<usize>,
     },
 
-    #[error("Trying to expand fixed index, index {index:?} old shape{old_shape:?} new shape {new_shape:?}")]
+    #[error("Trying to expand fixed index, index {index:?} old shape{old_shape:?} new shape {new_shape:?} ({e_name})")]
     FixedIndex {
         index: TensorIndex,
         old_shape: Shape,
@@ -407,7 +408,7 @@ pub enum ExpandError {
     },
 
     #[error(
-        "Trying to expand concat axis, index {axis} old shape{old_shape:?} new shape {new_shape:?}"
+        "Trying to expand concat axis, index {axis} old shape{old_shape:?} new shape {new_shape:?} ({e_name})"
     )]
     ConcatAxis {
         axis: usize,
@@ -415,16 +416,16 @@ pub enum ExpandError {
         new_shape: Shape,
     },
 
-    #[error("Inputs that should have same batching have different batchings, {batch_shapes:?} {circuit:?}")]
+    #[error("Inputs that should have same batching have different batchings, {batch_shapes:?} {circuit:?} ({e_name})")]
     InconsistentBatches {
         batch_shapes: Vec<Shape>,
         circuit: CircuitRc,
     },
 
-    #[error("trying to expand node, unknown variant {variant}")]
+    #[error("trying to expand node, unknown variant {variant} ({e_name})")]
     NodeUnhandledVariant { variant: String },
 
-    #[error("Not currently supported! old_shape={old_shape:?} != new_shape={new_shape:?}\nold_spec_circuit={old_spec_circuit:?} new_spec_circuit={new_spec_circuit:?}")]
+    #[error("Not currently supported! old_shape={old_shape:?} != new_shape={new_shape:?}\nold_spec_circuit={old_spec_circuit:?} new_spec_circuit={new_spec_circuit:?} ({e_name})")]
     ModuleSpecCircuitChangedShapeInExpand {
         old_shape: Shape,
         new_shape: Shape,
@@ -432,13 +433,13 @@ pub enum ExpandError {
         new_spec_circuit: CircuitRc,
     },
 
-    #[error("old_symbol={old_symbol:?} new_circuit={new_circuit:?}")]
+    #[error("old_symbol={old_symbol:?} new_circuit={new_circuit:?} ({e_name})")]
     ModuleSpecArgSymbolReplacedWithNonSymbol {
         old_symbol: Symbol,
         new_circuit: CircuitRc,
     },
 
-    #[error("Not currently supported! old_shape={old_shape:?} != new_shape={new_shape:?}\nold_symbol={old_symbol:?} new_symbol={new_symbol:?}")]
+    #[error("Not currently supported! old_shape={old_shape:?} != new_shape={new_shape:?}\nold_symbol={old_symbol:?} new_symbol={new_symbol:?} ({e_name})")]
     ModuleSpecArgSymbolChangedShapeInExpand {
         old_shape: Shape,
         new_shape: Shape,
@@ -446,7 +447,7 @@ pub enum ExpandError {
         new_symbol: Symbol,
     },
 
-    #[error("node_rank={node_rank} < symbol_rank={symbol_rank}, arg_spec={arg_spec:?} node_shape={node_shape:?} spec_circuit={spec_circuit:?}")]
+    #[error("node_rank={node_rank} < symbol_rank={symbol_rank}, arg_spec={arg_spec:?} node_shape={node_shape:?} spec_circuit={spec_circuit:?} ({e_name})")]
     ModuleRankReduced {
         node_rank: usize,
         symbol_rank: usize,
@@ -455,7 +456,7 @@ pub enum ExpandError {
         spec_circuit: CircuitRc,
     },
 
-    #[error("node_rank={node_rank} > symbol_rank={symbol_rank} (which indicates batching) and arg_spec={arg_spec:?} spec_circuit={spec_circuit:?}")]
+    #[error("node_rank={node_rank} > symbol_rank={symbol_rank} (which indicates batching) and arg_spec={arg_spec:?} spec_circuit={spec_circuit:?} ({e_name})")]
     ModuleTriedToBatchUnbatchableInput {
         node_rank: usize,
         symbol_rank: usize,
@@ -463,7 +464,7 @@ pub enum ExpandError {
         spec_circuit: CircuitRc,
     },
 
-    #[error("node_shape={node_shape:?} symbol_shape={symbol_shape:?} arg_spec={arg_spec:?} spec_circuit={spec_circuit:?}")]
+    #[error("node_shape={node_shape:?} symbol_shape={symbol_shape:?} arg_spec={arg_spec:?} spec_circuit={spec_circuit:?} ({e_name})")]
     ModuleTriedToExpandUnexpandableInput {
         node_shape: Shape,
         symbol_shape: Shape,
