@@ -10,7 +10,7 @@ use std::{
 };
 
 use anyhow::{anyhow, bail, Context, Result};
-use circuit_utils::toposort_circuit;
+use circuit_utils::{toposort_circuit, OperatorPriority};
 pub use computational_node::{
     flat_concat, flat_concat_back, Add, Concat, Conv, Einsum, Index, Rearrange, Scatter,
 };
@@ -22,6 +22,7 @@ use macro_rules_attribute::apply;
 pub use module::{Module, ModuleArgSpec, ModuleSpec};
 use num_bigint::BigUint;
 use print::PrintOptions;
+use print_html::{PrintHtmlOptions, PrintOptionsBase};
 use pyo3::{
     exceptions::{self, PyValueError},
     prelude::*,
@@ -59,6 +60,7 @@ pub mod nrc;
 pub mod opaque_iterative_matcher;
 pub mod parsing;
 pub mod print;
+pub mod print_html;
 pub mod set_symbolic_size;
 pub mod variable_nodes;
 
@@ -195,6 +197,11 @@ pub trait CircuitNode: CircuitNodeInit + CircuitNodeSelfOnlyHash {
 
     // TODO: add more map utils for non_free as needed
 
+    fn num_children(&self) -> usize {
+        // not exactly peak efficiency...
+        self.children().count()
+    }
+
     fn map_children<F>(&self, mut f: F) -> Result<Self>
     where
         Self: Sized,
@@ -561,7 +568,14 @@ pub trait CircuitNode: CircuitNodeInit + CircuitNodeSelfOnlyHash {
 }
 
 pub trait CircuitNodeAutoName: CircuitNode {
+    /// There are three kinds of automatically generated names:
+    /// - infix, like Add and Einsum. Parenthesis are added around children which would have had wrong priority
+    /// - functions, like GeneralFunction and Cumulant, which use the standard function notation f(x1, ... xn)
+    /// - postfix, like Rearrange and index, which behave like the unary operator "!". Parenthesis are added if needed
+    /// utils to add the parenthesis can be found in circuit_utils.rs
     fn auto_name(&self, name: Option<String>) -> Option<String>;
+
+    const PRIORITY: OperatorPriority = OperatorPriority::NotOperator {};
     const CHILD_NAME_MAX_LEN: usize = 100; // TODO: fix me!
     /// we shorten children names on multi child nodes
     fn shorten_child_name(name: &str) -> String {
@@ -1202,25 +1216,25 @@ impl CachedCircuitInfo {
 #[base_exception(PyValueError)]
 #[derive(Error, Debug, Clone)]
 pub enum ConstructError {
-    #[error("{string}")]
+    #[error("{string} ({e_name})")]
     NotSupportedYet { string: String },
 
-    #[error("Conv requires stride and padding to be full size, got stride len {stride} padding len {padding}")]
+    #[error("Conv requires stride and padding to be full size, got stride len {stride} padding len {padding} ({e_name})")]
     ConvStridePaddingNotFull { stride: usize, padding: usize },
 
-    #[error("Conv input wrong shape")]
+    #[error("Conv input wrong shape ({e_name})")]
     ConvInputWrongShape {},
 
-    #[error("Conv input wrong shape")]
+    #[error("Conv input wrong shape ({e_name})")]
     ConvFilterWrongShape {},
 
-    #[error("Conv kernel shape must be all odd, got")]
+    #[error("Conv kernel shape must be all odd, got ({e_name})")]
     ConvFilterMustBeOddLength { shape: Shape },
 
-    #[error("Conv stride must evenly divide")]
+    #[error("Conv stride must evenly divide ({e_name})")]
     ConvStrideMustDivide { shape: Shape, stride: Vec<usize> },
 
-    #[error("Conv groups ({groups}) must divide input channels ({in_channels}) and output channels ({out_channels})")]
+    #[error("Conv groups ({groups}) must divide input channels ({in_channels}) and output channels ({out_channels}) ({e_name})")]
     ConvGroupsMustDivide {
         groups: usize,
         in_channels: usize,
@@ -1228,46 +1242,46 @@ pub enum ConstructError {
     },
 
     #[error(
-        "Conv input and filter must have same number of input channels, got {input} and {filter}"
+        "Conv input and filter must have same number of input channels, got {input} and {filter} ({e_name})"
     )]
     ConvInputFilterDifferentNumInputChannels { input: usize, filter: usize },
-    #[error("DiscreteVar doesn't have leading 'samples' dim")]
+    #[error("DiscreteVar doesn't have leading 'samples' dim ({e_name})")]
     DiscreteVarNoSamplesDim {},
 
-    #[error("DiscreteVar samples dim doesn't match probs, {node} vs {probs}")]
+    #[error("DiscreteVar samples dim doesn't match probs, {node} vs {probs} ({e_name})")]
     DiscreteVarWrongSamplesDim { node: usize, probs: usize },
 
-    #[error("DiscreteVar probs must be 1d with length matching samples axis 0, got probs of shape {shape:?}")]
+    #[error("DiscreteVar probs must be 1d with length matching samples axis 0, got probs of shape {shape:?} ({e_name})")]
     DiscreteVarProbsMustBe1d { shape: Shape },
 
-    #[error("StoredCumulantVar needs first 2 cumulants specified")]
+    #[error("StoredCumulantVar needs first 2 cumulants specified ({e_name})")]
     StoredCumulantVarNeedsMeanVariance {},
 
-    #[error("StoredCumulantVar invalid cumulant number {number}")]
+    #[error("StoredCumulantVar invalid cumulant number {number} ({e_name})")]
     StoredCumulantVarInvalidCumulantNumber { number: usize },
 
-    #[error("StoredCumulantVar cumulant number {cumulant_number} needs to be base shape, {base_shape:?} times cumulant number, got {cumulant_shape:?}")]
+    #[error("StoredCumulantVar cumulant number {cumulant_number} needs to be base shape, {base_shape:?} times cumulant number, got {cumulant_shape:?} ({e_name})")]
     StoredCumulantVarCumulantWrongShape {
         base_shape: Shape,
         cumulant_shape: Shape,
         cumulant_number: usize,
     },
 
-    #[error("actual_num_children={actual_num_children} != expected_num_children_based_on_axes={expected_num_children_based_on_axes}\neinsum_name={einsum_name:?}")]
+    #[error("actual_num_children={actual_num_children} != expected_num_children_based_on_axes={expected_num_children_based_on_axes}\neinsum_name={einsum_name:?} ({e_name})")]
     EinsumWrongNumChildren {
         actual_num_children: usize,
         expected_num_children_based_on_axes: usize,
         einsum_name: Option<String>,
     },
 
-    #[error("len_axes={len_axes} != child_len_shape={child_len_shape}\neinsum_name={einsum_name:?} child_circuit={child_circuit:?}")]
+    #[error("len_axes={len_axes} != child_len_shape={child_len_shape}\neinsum_name={einsum_name:?} child_circuit={child_circuit:?} ({e_name})")]
     EinsumLenShapeDifferentFromAxes {
         len_axes: usize,
         child_len_shape: usize,
         einsum_name: Option<String>,
         child_circuit: CircuitRc,
     },
-    #[error("new_size={new_size}!=existing_size={existing_size} for axis={axis}\neinsum_name={einsum_name:?}, size introduced with child_circuit={child_circuit:?}")]
+    #[error("new_size={new_size}!=existing_size={existing_size} for axis={axis}\neinsum_name={einsum_name:?}, size introduced with child_circuit={child_circuit:?} ({e_name})")]
     EinsumAxisSizeDifferent {
         new_size: SymbolicSizeProduct,
         existing_size: SymbolicSizeProduct,
@@ -1275,48 +1289,48 @@ pub enum ConstructError {
         einsum_name: Option<String>,
         child_circuit: CircuitRc,
     },
-    #[error("circuit_name={circuit_name:?} all_input_axes={all_input_axes:?} output_axes={output_axes:?}")]
+    #[error("circuit_name={circuit_name:?} all_input_axes={all_input_axes:?} output_axes={output_axes:?} ({e_name})")]
     EinsumOutputNotSubset {
         circuit_name: Option<String>,
         all_input_axes: HashSet<u8>,
         output_axes: EinsumAxes,
     },
 
-    #[error("Rearrange takes different input shape, shape: {shape:?} spec: {spec:?}")]
+    #[error("Rearrange takes different input shape, shape: {shape:?} spec: {spec:?} ({e_name})")]
     RearrangeWrongInputShape { spec: RearrangeSpec, shape: Shape },
 
-    #[error("Wrong input shapes for GeneralFunction {input_shapes:?} {gf_spec:?}")]
+    #[error("Wrong input shapes for GeneralFunction {input_shapes:?} {gf_spec:?} ({e_name})")]
     GeneralFunctionWrongInputShape {
         gf_spec: GeneralFunctionSpec,
         input_shapes: Vec<Shape>,
     },
 
-    #[error("Passed python object isn't instance of GeneralFunctionSpec abstract class, ob={ob}")]
+    #[error("Passed python object isn't instance of GeneralFunctionSpec abstract class, ob={ob} ({e_name})")]
     GeneralFunctionPyNotInstance { ob: PyObject },
 
-    #[error("Concat requires at least one node")]
+    #[error("Concat requires at least one node ({e_name})")]
     ConcatZeroNodes {},
 
-    #[error("Concat nodes have different shapes {shapes:?}")]
+    #[error("Concat nodes have different shapes {shapes:?} ({e_name})")]
     ConcatShapeDifferent { shapes: Vec<Shape> },
 
-    #[error("axis out of bounds: {axis} vs {node_rank}")]
+    #[error("axis out of bounds: {axis} vs {node_rank} ({e_name})")]
     AxisOutOfBounds { axis: i64, node_rank: usize },
 
-    #[error("Scatter shape wrong, index: {index_shape:?} child: {shape:?} {index:?}")]
+    #[error("Scatter shape wrong, index: {index_shape:?} child: {shape:?} {index:?} ({e_name})")]
     ScatterShapeWrong {
         index: TensorIndex,
         shape: Shape,
         index_shape: Shape,
     },
 
-    #[error("This scatter index not supported yet, {index:?}")]
+    #[error("This scatter index not supported yet, {index:?} ({e_name})")]
     ScatterIndexTypeUnimplemented { index: TensorIndex },
 
-    #[error("Unknown GeneralFunction name {spec_name}")]
+    #[error("Unknown GeneralFunction name {spec_name} ({e_name})")]
     UnknownGeneralFunction { spec_name: String },
 
-    #[error("Module wrong number of children, expected {expected} got {got}\narg_specs={arg_specs:?} nodes={nodes:?}")]
+    #[error("Module wrong number of children, expected {expected} got {got}\narg_specs={arg_specs:?} nodes={nodes:?} ({e_name})")]
     ModuleWrongNumberChildren {
         expected: usize,
         got: usize,
@@ -1325,50 +1339,52 @@ pub enum ConstructError {
     },
 
     #[error(
-        "Module got unknown keyword argument, {argument}, all_module_inputs={all_module_inputs:?}"
+        "Module got unknown keyword argument, {argument}, all_module_inputs={all_module_inputs:?} ({e_name})"
     )]
     ModuleUnknownArgument {
         argument: String,
         all_module_inputs: Vec<String>,
     },
 
-    #[error("missing_arguments={missing_arguments:?}")]
+    #[error("missing_arguments={missing_arguments:?} ({e_name})")]
     ModuleMissingNames { missing_arguments: Vec<String> },
 
-    #[error("Module extract: not all leaves present in circuit, {subcirc:?}")]
+    #[error("Module extract: not all leaves present in circuit, {subcirc:?} ({e_name})")]
     ModuleExtractNotPresent { subcirc: CircuitRc },
 
-    #[error("symbols_named_none={symbols_named_none:?}")]
+    #[error("symbols_named_none={symbols_named_none:?} ({e_name})")]
     ModuleSomeArgsNamedNone { symbols_named_none: Vec<Symbol> },
 
-    #[error("dup_names={dup_names:?}")]
+    #[error("dup_names={dup_names:?} ({e_name})")]
     ModuleArgsDupNames { dup_names: HashMap<String, usize> },
 
-    #[error("spec_circuit={spec_circuit:?} missing_symbols={missing_symbols:?}")]
+    #[error("spec_circuit={spec_circuit:?} missing_symbols={missing_symbols:?} ({e_name})")]
     ModuleSomeArgsNotPresent {
         spec_circuit: CircuitRc,
         missing_symbols: HashSet<Symbol>,
     },
 
-    #[error("actual_circuit={actual_circuit:?}\n(module children are: spec,sym_0,inp_0,sym_1,inp_1,...,sym_n,inp_n)")]
+    #[error("actual_circuit={actual_circuit:?}\n(module children are: spec,sym_0,inp_0,sym_1,inp_1,...,sym_n,inp_n) ({e_name})")]
     ModuleExpectedSymbol { actual_circuit: CircuitRc },
 
-    #[error("orig_module={orig_module:?} got_instead_of_symbol={got_instead_of_symbol:?}")]
+    #[error(
+        "orig_module={orig_module:?} got_instead_of_symbol={got_instead_of_symbol:?} ({e_name})"
+    )]
     ModuleExpectedSymbolOnMap {
         orig_module: Module,
         got_instead_of_symbol: CircuitRc,
     },
 
-    #[error("Named axis higher than rank")]
+    #[error("Named axis higher than rank ({e_name})")]
     NamedAxisAboveRank {},
 
-    #[error("Failed to construct equivalent explicitly computable circuit")]
+    #[error("Failed to construct equivalent explicitly computable circuit ({e_name})")]
     NoEquivalentExplicitlyComputable {},
 
-    #[error("ndim={ndim}")]
+    #[error("ndim={ndim} ({e_name})")]
     UnflattenButNDimNot1 { ndim: usize },
 
-    #[error("name_prefix='{name_prefix}' module={module:?}")]
+    #[error("name_prefix='{name_prefix}' module={module:?} ({e_name})")]
     ModulePassedNamePrefixAndUseSelfNameAsPrefix { name_prefix: String, module: Module },
 }
 
@@ -1382,20 +1398,20 @@ const INTERNAL_EVAL_ERROR_MESSAGE: &str = "this is likely an internal error (ear
 #[derive(Error, Debug, Clone)]
 pub enum TensorEvalError {
     #[error(
-        "reached not explicitly computable. {}\ncircuit={circuit:?})",
+        "reached not explicitly computable. {}\ncircuit={circuit:?}) ({e_name})",
         INTERNAL_EVAL_ERROR_MESSAGE
     )]
     NotExplicitlyComputableInternal { circuit: CircuitRc },
-    #[error("not explicitly computable: {circuit:?})")]
+    #[error("not explicitly computable: {circuit:?}) ({e_name})")]
     NotExplicitlyComputable { circuit: CircuitRc },
-    #[error("not constant: {circuit:?})")]
+    #[error("not constant: {circuit:?}) ({e_name})")]
     NotConstant { circuit: CircuitRc },
     #[error(
-        "You can't directly evaluate  module. {}\nmodule={module:?})",
+        "You can't directly evaluate  module. {}\nmodule={module:?}) ({e_name})",
         INTERNAL_EVAL_ERROR_MESSAGE
     )]
     ModulesCantBeDirectlyEvalutedInternal { module: Module },
-    #[error("incompatible dtype circ:{circ:?} passed:{passed:?}")]
+    #[error("incompatible dtype circ:{circ:?} passed:{passed:?} ({e_name})")]
     DeviceDtypeError {
         circ: TorchDeviceDtypeOp,
         passed: TorchDeviceDtypeOp,
@@ -1574,6 +1590,7 @@ where
 }
 
 // TODO: maybe this already exists?
+#[pyfunction]
 pub fn all_children(circuit: CircuitRc) -> HashSet<CircuitRc> {
     let mut seen: HashSet<CircuitRc> = HashSet::default();
     fn recurse(circ: CircuitRc, seen: &mut HashSet<CircuitRc>) {
@@ -1961,9 +1978,13 @@ impl PyCircuitBase {
     }
 
     #[args(options = "Default::default()")]
-    fn print(&self, options: PrintOptions) -> Result<()> {
-        rr_util::python_println!("{}", self.repr(options)?);
-        Ok(())
+    fn print(&self, options: PrintOptionsBase) -> Result<()> {
+        options.print(vec![(*self.0).crc()])
+    }
+
+    #[args(options = "Default::default()")]
+    fn print_html(&self, options: PrintHtmlOptions) -> Result<()> {
+        options.print(vec![(*self.0).crc()])
     }
 
     fn numel(&self) -> BigUint {
@@ -1998,6 +2019,10 @@ impl PyCircuitBase {
         self.0.map_children(|child| pycall!(f, (child,), anyhow))
     }
 
+    fn num_children(&self) -> usize {
+        self.0.num_children()
+    }
+
     fn total_arrayconstant_size(&self) -> BigUint {
         total_arrayconstant_size(self.0.clone())
     }
