@@ -14,8 +14,8 @@ use uuid::{uuid, Uuid};
 use crate::{
     all_children, circuit_node_auto_impl, circuit_node_extra_impl,
     circuit_node_private::{CircuitNodeHashItems, CircuitNodePrivate, CircuitNodeSetNonHashInfo},
-    circuit_utils::replace_nodes,
-    deep_map_op_context_preorder_stoppable, deep_map_unwrap,
+    circuit_utils::OperatorPriority,
+    deep_map, deep_map_op_context_preorder_stoppable, deep_map_unwrap,
     expand_node::{replace_expand_bottom_up, ExpandError},
     new_rc_unwrap,
     prelude::*,
@@ -26,9 +26,9 @@ use crate::{
 #[pyclass]
 #[derive(Debug, Clone, Hash, PartialEq, Eq)]
 pub struct ModuleSpec {
-    #[pyo3(get)]
+    #[pyo3(get, set)]
     pub circuit: CircuitRc,
-    #[pyo3(get)]
+    #[pyo3(get, set)]
     pub arg_specs: Vec<ModuleArgSpec>,
 }
 
@@ -154,21 +154,25 @@ impl ModuleSpec {
                 x.clone()
             })
         }
-        let result = replace_nodes(expanded_shape, &node_mapping);
+        // we're replacing leaves so no need to do anything with stoppable preorder
+        let result = deep_map(expanded_shape, |circ| {
+            Ok(node_mapping.get(&circ.info().hash).cloned().unwrap_or(circ))
+        })
+        .with_context(|| {
+            format!(
+                concat!(
+                    "replacing symbols with nodes (which are not necessarily symbols)",
+                    " failed in ModuleSpec substitute\n",
+                    "(perhaps you replaced the input symbols to a nested module?)\n",
+                    "module_spec={:?}\nnodes={:?}"
+                ),
+                self, nodes
+            )
+        })?;
         MODULE_EXPANSIONS.with(|cache| cache.borrow_mut().insert(key, result.clone()));
         Ok(result)
     }
 
-    // pub fn compute_hash(&self) -> HashBytes {
-    //     let mut hasher = blake3::Hasher::new();
-    //     hasher.update(&self.circuit.info().hash);
-    //     for arg_spec in &self.arg_specs {
-    //         hasher.update(&[arg_spec.batchable as u8, arg_spec.expandable as u8]);
-    //         hasher.update(&arg_spec.symbol.info().hash);
-    //     }
-    //     hasher.finalize().into()
-    // }
-
     pub fn compute_non_children_hash(&self, hasher: &mut blake3::Hasher) {
         for arg_spec in &self.arg_specs {
             // this is fine because each item is fixed size and we delimit with node hashs (which are uu)
@@ -180,15 +184,20 @@ impl ModuleSpec {
     where
         F: FnMut(CircuitRc) -> Result<CircuitRc>,
     {
-        ModuleSpec::new(f(self.circuit.clone())?, self.arg_specs.clone(), false)
+        Ok(ModuleSpec {
+            circuit: f(self.circuit.clone())?,
+            arg_specs: self.arg_specs.clone(),
+        })
     }
     pub fn map_circuit_unwrap<F>(&self, mut f: F) -> Self
     where
         F: FnMut(CircuitRc) -> CircuitRc,
     {
-        ModuleSpec::new(f(self.circuit.clone()), self.arg_specs.clone(), false).unwrap()
+        ModuleSpec {
+            circuit: f(self.circuit.clone()),
+            arg_specs: self.arg_specs.clone(),
+        }
     }
-    // TODO: add no_check_spec versions as needed!
 }
 
 #[pyfunction]
@@ -206,34 +215,36 @@ pub fn all_free(circuit: CircuitRc) -> Vec<Symbol> {
 #[pymethods]
 impl ModuleSpec {
     #[new]
-    #[args(check_unique_arg_names = "true")]
+    #[args(check_all_args_present = "true", check_unique_arg_names = "true")]
     pub fn new(
         circuit: CircuitRc,
         arg_specs: Vec<ModuleArgSpec>,
+        check_all_args_present: bool,
         check_unique_arg_names: bool,
     ) -> Result<Self> {
-        let all_symbols = arg_specs.iter().map(|x| x.symbol.crc()).collect();
-        let children = all_children(circuit.clone());
-        if !children.is_superset(&all_symbols) {
-            bail!(ConstructError::ModuleSomeArgsNotPresent {
-                spec_circuit: circuit,
-                missing_symbols: all_symbols
-                    .difference(&children)
-                    .map(|x| x.as_symbol_unwrap().clone())
-                    .collect(),
-            })
-        }
-
         let out = Self { circuit, arg_specs };
+        if check_all_args_present {
+            out.check_all_args_present()?;
+        }
         if check_unique_arg_names {
             out.check_unique_arg_names()?;
         }
         Ok(out)
     }
 
-    #[staticmethod]
-    fn new_no_check_args(circuit: CircuitRc, arg_specs: Vec<ModuleArgSpec>) -> Self {
-        Self { circuit, arg_specs }
+    fn check_all_args_present(&self) -> Result<()> {
+        let all_symbols = self.arg_specs.iter().map(|x| x.symbol.crc()).collect();
+        let children = all_children(self.circuit.clone());
+        if !children.is_superset(&all_symbols) {
+            bail!(ConstructError::ModuleSomeArgsNotPresent {
+                spec_circuit: self.circuit.clone(),
+                missing_symbols: all_symbols
+                    .difference(&children)
+                    .map(|x| x.as_symbol_unwrap().clone())
+                    .collect(),
+            })
+        }
+        Ok(())
     }
 
     fn check_unique_arg_names(&self) -> Result<()> {
@@ -273,7 +284,8 @@ impl ModuleSpec {
             .into_iter()
             .map(|x| ModuleArgSpec::new(x, true, true))
             .collect();
-        let out = Self::new(circuit, arg_specs, false).expect("method should guarantee valid");
+        let out =
+            Self::new(circuit, arg_specs, true, false).expect("method should guarantee valid");
         if check_unique_arg_names {
             // check after instead of in new so we can .expect on other errors in new
             out.check_unique_arg_names()?;
@@ -283,11 +295,11 @@ impl ModuleSpec {
     }
 
     #[staticmethod]
-    #[args(require_all_args = "true", check_unique_arg_names = "true")]
+    #[args(check_all_args_present = "true", check_unique_arg_names = "true")]
     pub fn new_extract(
         circuit: CircuitRc,
         arg_specs: Vec<(CircuitRc, ModuleArgSpec)>,
-        require_all_args: bool,
+        check_all_args_present: bool,
         check_unique_arg_names: bool,
     ) -> Result<Self> {
         let mut new_arg_specs: Vec<Option<ModuleArgSpec>> = vec![None; arg_specs.len()];
@@ -318,7 +330,7 @@ impl ModuleSpec {
             &mut Default::default(),
         )
         .unwrap_or(circuit);
-        let new_arg_specs: Vec<ModuleArgSpec> = if require_all_args {
+        let new_arg_specs: Vec<ModuleArgSpec> = if check_all_args_present {
             let er = new_arg_specs
                 .iter()
                 .cloned()
@@ -332,13 +344,14 @@ impl ModuleSpec {
         } else {
             new_arg_specs
                 .into_iter()
+                // TODO: maybe instead of filter we're supposed to just have missing module args?
                 .filter(|z| z.is_some())
                 .collect::<Option<Vec<_>>>()
                 .unwrap()
         };
         // maybe we should use no_check_args for speed
-        let out =
-            Self::new(spec_circuit, new_arg_specs, false).expect("method should guarantee valid");
+        let out = Self::new(spec_circuit, new_arg_specs, check_all_args_present, false)
+            .expect("method should guarantee valid");
         if check_unique_arg_names {
             // check after instead of in new so we can .expect on other errors in new
             out.check_unique_arg_names()?;
@@ -346,6 +359,7 @@ impl ModuleSpec {
         Ok(out)
     }
 
+    // TODO: add some naming options maybe
     pub fn resize(&self, shapes: Vec<Shape>) -> Result<Self> {
         let arg_specs: Vec<ModuleArgSpec> = zip(&self.arg_specs, shapes)
             .map(|(arg_spec, shape)| ModuleArgSpec {
@@ -354,14 +368,15 @@ impl ModuleSpec {
                 expandable: arg_spec.expandable,
             })
             .collect();
-        let circuit = replace_expand_bottom_up(self.circuit.clone(), |c| {
-            self.arg_specs
-                .iter()
-                .position(|x| x.symbol.info().hash == c.info().hash)
-                .map(|p| arg_specs[p].symbol.crc())
-        })?;
-        // maybe we should use no_check_args for speed
-        Ok(Self::new(circuit, arg_specs, false).expect("method should guarantee valid"))
+
+        let circuit = self
+            .substitute(
+                &arg_specs.iter().map(|x| x.symbol.crc()).collect(),
+                None,
+                None,
+            )
+            .context("substitute failed from resize")?;
+        Ok(Self { circuit, arg_specs })
     }
 
     pub fn __repr__(&self) -> String {
@@ -582,7 +597,10 @@ impl CircuitNode for Module {
 
         Self::try_new(
             nodes,
-            ModuleSpec::new(new_spec_circ, arg_specs, false)?,
+            ModuleSpec {
+                circuit: new_spec_circ,
+                arg_specs,
+            },
             self.name.clone(),
         )
     }
@@ -613,6 +631,8 @@ impl CircuitNode for Module {
 }
 
 impl CircuitNodeAutoName for Module {
+    const PRIORITY: OperatorPriority = OperatorPriority::Function {};
+
     fn auto_name(&self, name: Option<String>) -> Option<String> {
         name.or_else(|| {
             if self.children().any(|x| x.name().is_none()) {
@@ -620,21 +640,14 @@ impl CircuitNodeAutoName for Module {
             } else {
                 Some(
                     self.spec.circuit.name_cloned().unwrap()
-                        + " "
+                        + "("
                         + &self
-                            .spec
-                            .arg_specs
+                            .nodes
                             .iter()
-                            .zip(&self.nodes)
-                            .map(|(arg_spec, node)| {
-                                format!(
-                                    "{}={}",
-                                    Self::shorten_child_name(arg_spec.symbol.name().unwrap()),
-                                    Self::shorten_child_name(node.name().unwrap())
-                                )
-                            })
+                            .map(|node| node.name_cloned().unwrap())
                             .collect::<Vec<String>>()
-                            .join(", "),
+                            .join(", ")
+                        + ")",
                 )
             }
         })
@@ -687,6 +700,17 @@ impl Module {
         };
         Ok(self.substitute(name_prefix, name_suffix))
     }
+
+    // TODO: add some naming options maybe
+    pub fn conform_to_input_shapes(&self) -> Self {
+        Self::new(
+            self.nodes.clone(),
+            self.spec
+                .resize(self.nodes.iter().map(|x| x.shape().clone()).collect())
+                .expect("constructor should ensure this works"),
+            self.name_cloned(),
+        )
+    }
 }
 
 impl Module {
@@ -709,10 +733,19 @@ impl Module {
 #[pyfunction]
 pub fn inline_all_modules(circuit: CircuitRc) -> CircuitRc {
     deep_map_unwrap(circuit, |c| match &**c {
-        Circuit::Module(mn) => mn.substitute(None, None), /* spec.circuit already inlined due to recursive calls */
+        Circuit::Module(mn) => mn.substitute(None, None),
         _ => c.clone(),
     })
 }
+
+#[pyfunction]
+pub fn conform_all_modules(circuit: CircuitRc) -> CircuitRc {
+    deep_map_unwrap(circuit, |c| match &**c {
+        Circuit::Module(mn) => mn.conform_to_input_shapes().rc(),
+        _ => c.clone(),
+    })
+}
+
 use std::cell::RefCell;
 // TODO: maybe cache check_unique_arg_names here?
 thread_local! {
