@@ -26,7 +26,7 @@ use crate::{
 #[pyclass]
 #[derive(Clone, Debug)]
 pub struct Parser {
-    #[pyo3(get)]
+    #[pyo3(get, set)]
     pub reference_circuits: HashMap<String, CircuitRc>,
     #[pyo3(get, set)]
     pub tensors_as_random: bool,
@@ -34,6 +34,10 @@ pub struct Parser {
     pub tensors_as_random_device_dtype: TorchDeviceDtypeOp,
     #[pyo3(get, set)]
     pub on_repeat_check_info_same: bool,
+    #[pyo3(get, set)]
+    pub module_check_all_args_present: bool,
+    #[pyo3(get, set)]
+    pub module_check_unique_arg_names: bool,
 }
 
 impl Default for Parser {
@@ -43,6 +47,8 @@ impl Default for Parser {
             tensors_as_random: false,
             tensors_as_random_device_dtype: TorchDeviceDtypeOp::NONE,
             on_repeat_check_info_same: true,
+            module_check_all_args_present: true,
+            module_check_unique_arg_names: false,
         }
     }
 }
@@ -54,7 +60,9 @@ impl Parser {
         reference_circuits_by_name = "vec![]",
         tensors_as_random = "Parser::default().tensors_as_random",
         tensors_as_random_device_dtype = "Parser::default().tensors_as_random_device_dtype",
-        on_repeat_check_info_same = "Parser::default().on_repeat_check_info_same"
+        on_repeat_check_info_same = "Parser::default().on_repeat_check_info_same",
+        module_check_all_args_present = "Parser::default().module_check_all_args_present",
+        module_check_unique_arg_names = "Parser::default().module_check_unique_arg_names"
     )]
     #[new]
     pub fn new(
@@ -63,6 +71,8 @@ impl Parser {
         tensors_as_random: bool,
         tensors_as_random_device_dtype: TorchDeviceDtypeOp,
         on_repeat_check_info_same: bool,
+        module_check_all_args_present: bool,
+        module_check_unique_arg_names: bool,
     ) -> Result<Self> {
         let ref_circ_names = reference_circuits_by_name
             .into_iter()
@@ -99,6 +109,8 @@ impl Parser {
             tensors_as_random,
             tensors_as_random_device_dtype,
             on_repeat_check_info_same,
+            module_check_all_args_present,
+            module_check_unique_arg_names,
         })
     }
 
@@ -447,12 +459,32 @@ impl Parser {
             return Ok(already.clone());
         }
         let ps = &partial_circuits[ident];
-        let mut children: Vec<CircuitRc> = ps
+        let children: Vec<CircuitRc> = ps
             .children
             .iter()
             .map(|x| self.deep_convert_partial_circ(x, partial_circuits, context, tensor_cache))
             .collect::<Result<Vec<_>, _>>()?;
 
+        let result = self
+            .deep_convert_partial_circ_children(ident, ps, children, tensor_cache)
+            .with_context(|| {
+                format!(
+                    "in parse, failed to convert partial circuit with ident={:?}",
+                    ident
+                )
+            })?;
+
+        context.insert(ident.clone(), result.clone());
+        Ok(result)
+    }
+
+    fn deep_convert_partial_circ_children(
+        &self,
+        ident: &CircuitIdent,
+        ps: &PartialCirc,
+        mut children: Vec<CircuitRc>,
+        tensor_cache: &mut Option<TensorCacheRrfs>,
+    ) -> Result<CircuitRc> {
         let expected_k_children = |k| {
             if children.len() != k {
                 bail!(ParseError::WrongNumberChildren {
@@ -659,8 +691,13 @@ impl Parser {
 
                 Module::try_new(
                     nodes,
-                    ModuleSpec::new(spec_circuit, arg_specs, false)
-                        .context("spec construction failed in parse")?,
+                    ModuleSpec::new(
+                        spec_circuit,
+                        arg_specs,
+                        self.module_check_all_args_present,
+                        self.module_check_unique_arg_names,
+                    )
+                    .context("spec construction failed in parse")?,
                     ps.name.clone(),
                 )
                 .context("module construction failed in parse")?
@@ -735,7 +772,6 @@ impl Parser {
                 unimplemented!()
             }
         };
-        context.insert(ident.clone(), result.clone());
         Ok(result)
     }
 }
@@ -745,9 +781,9 @@ impl Parser {
 #[base_exception(PyValueError)]
 #[derive(Error, Debug, Clone)]
 pub enum ParseArgError {
-    #[error("circuit={circuit:?}")]
+    #[error("circuit={circuit:?} ({e_name})")]
     ReferenceCircuitByNameHasNoneName { circuit: CircuitRc },
 
-    #[error("dup_idents={dup_idents:?}")]
+    #[error("dup_idents={dup_idents:?} ({e_name})")]
     ReferenceCircuitDuplicateIdentifier { dup_idents: HashMap<String, usize> },
 }
