@@ -32,7 +32,7 @@ const ARROW: &'static str = "‣";
 const UP_ELBOW: &'static str = "└";
 const DOWN_ELBOW: &'static str = "┌";
 
-fn_struct!(pub CircuitToColor:Fn(circuit:CircuitRc)->usize);
+fn_struct!(pub CircuitToColorCode:Fn(circuit:CircuitRc)->usize);
 
 #[pyclass]
 #[derive(Clone)]
@@ -41,7 +41,7 @@ pub struct PrintOptions {
     pub bijection: bool,
     #[pyo3(get, set)]
     pub shape_only_when_necessary: bool,
-    pub term_at: Option<OpaqueIterativeMatcherVal>,
+    pub traversal: Option<OpaqueIterativeMatcherVal>,
     #[pyo3(get, set)]
     pub leaves_on_top: bool,
     #[pyo3(get, set)]
@@ -50,7 +50,7 @@ pub struct PrintOptions {
     pub print_size_threshold: Option<usize>,
     #[pyo3(get, set)]
     pub force_use_serial_numbers: bool,
-    pub colorer: Option<CircuitToColor>,
+    pub colorer: Option<CircuitToColorCode>,
 }
 
 impl Default for PrintOptions {
@@ -58,7 +58,7 @@ impl Default for PrintOptions {
         Self {
             bijection: true,
             shape_only_when_necessary: true,
-            term_at: None,
+            traversal: None,
             leaves_on_top: false,
             arrows: false,
             print_size_threshold: None,
@@ -125,6 +125,30 @@ impl TryFrom<char> for TerseBool {
     }
 }
 
+pub fn op_traversal_per(
+    op_traversal: Option<OpaqueIterativeMatcherVal>,
+    circ: CircuitRc,
+) -> Result<Option<Vec<Option<OpaqueIterativeMatcherVal>>>> {
+    let num_children = circ.num_children();
+    let new_traversal_per = if let Some(traversal) = op_traversal {
+        let updated = traversal
+            .opaque_match_iterate(circ.crc())?
+            .updated
+            .unwrap_or_else(|| Some(traversal).into());
+        if updated.all_finished() {
+            return Ok(None);
+        }
+        updated
+            .per_child_with_term(num_children)
+            .into_iter()
+            .map(Some)
+            .collect()
+    } else {
+        vec![None; num_children]
+    };
+    Ok(Some(new_traversal_per))
+}
+
 #[pymethods]
 impl PrintOptions {
     // new function for rust and python (in rust for validation)
@@ -132,7 +156,7 @@ impl PrintOptions {
     #[args(
         bijection = "PrintOptions::default().bijection",
         shape_only_when_necessary = "PrintOptions::default().shape_only_when_necessary",
-        term_at = "None",
+        traversal = "None",
         leaves_on_top = "PrintOptions::default().leaves_on_top",
         arrows = "PrintOptions::default().arrows",
         print_size_threshold = "None",
@@ -142,17 +166,17 @@ impl PrintOptions {
     pub fn new(
         bijection: bool,
         shape_only_when_necessary: bool,
-        term_at: Option<OpaqueIterativeMatcherVal>,
+        traversal: Option<OpaqueIterativeMatcherVal>,
         leaves_on_top: bool,
         arrows: bool,
         print_size_threshold: Option<usize>,
         force_use_serial_numbers: bool,
-        colorer: Option<CircuitToColor>,
+        colorer: Option<CircuitToColorCode>,
     ) -> Result<Self> {
         let result = Self {
             bijection,
             shape_only_when_necessary,
-            term_at,
+            traversal,
             leaves_on_top,
             arrows,
             print_size_threshold,
@@ -166,7 +190,7 @@ impl PrintOptions {
     #[args(
         bijection = "Default::default()",
         shape_only_when_necessary = "Default::default()",
-        term_at = "Default::default()",
+        traversal = "Default::default()",
         leaves_on_top = "Default::default()",
         arrows = "Default::default()",
         print_size_threshold = "Default::default()",
@@ -177,12 +201,12 @@ impl PrintOptions {
         &self,
         bijection: MaybeNotSet<bool>,
         shape_only_when_necessary: MaybeNotSet<bool>,
-        term_at: MaybeNotSet<Option<OpaqueIterativeMatcherVal>>,
+        traversal: MaybeNotSet<Option<OpaqueIterativeMatcherVal>>,
         leaves_on_top: MaybeNotSet<bool>,
         arrows: MaybeNotSet<bool>,
         print_size_threshold: MaybeNotSet<Option<usize>>,
         force_use_serial_numbers: MaybeNotSet<bool>,
-        colorer: MaybeNotSet<Option<CircuitToColor>>,
+        colorer: MaybeNotSet<Option<CircuitToColorCode>>,
     ) -> Result<Self> {
         let cloned = self.clone();
         Self::new(
@@ -190,7 +214,7 @@ impl PrintOptions {
             shape_only_when_necessary
                 .0
                 .unwrap_or(cloned.shape_only_when_necessary),
-            term_at.0.unwrap_or(cloned.term_at),
+            traversal.0.unwrap_or(cloned.traversal),
             leaves_on_top.0.unwrap_or(cloned.leaves_on_top),
             arrows.0.unwrap_or(cloned.arrows),
             print_size_threshold
@@ -220,7 +244,7 @@ impl PrintOptions {
         Self {
             bijection: false,
             shape_only_when_necessary: false,
-            term_at: end_depth.map(|x| OpaqueIterativeMatcherVal::for_end_depth(x)),
+            traversal: end_depth.map(|x| OpaqueIterativeMatcherVal::for_end_depth(x)),
             ..Default::default()
         }
         .validate_ret()
@@ -231,8 +255,10 @@ impl PrintOptions {
         if self.bijection && self.leaves_on_top {
             bail!(anyhow!("bijection print cant have leaves on top"))
         }
-        if self.bijection && self.term_at.is_some() {
-            bail!(anyhow!("bijection print cant terminate early"))
+        if self.bijection && self.traversal.is_some() {
+            bail!(anyhow!(
+                "bijection print cant terminate early, so traversal must be None"
+            ))
         }
         Ok(())
     }
@@ -308,7 +334,7 @@ impl PrintOptions {
                     if self.bijection {
                         gf.spec
                             .serialize()
-                            .context("failed to get spec serialize in print")?
+                            .context("failed to get general function spec serialize in print")?
                             .unwrap_or_else(|| format!("{} [NOT_SERIALIZABLE]", gf.spec.name()))
                     } else {
                         gf.spec.name()
@@ -322,20 +348,30 @@ impl PrintOptions {
                     }
                 }
                 Circuit::Module(mn) => {
-                    // print out out arg info, everything else in children
-                    // TODO: less terse?
-                    mn.spec
-                        .arg_specs
-                        .iter()
-                        .map(|arg_spec| {
-                            format!(
-                                "{}{}",
-                                TerseBool(arg_spec.batchable),
-                                TerseBool(arg_spec.expandable)
-                            )
-                        })
-                        .collect::<Vec<_>>()
-                        .join(",")
+                    if mn.spec.arg_specs.len() >= 3
+                        && mn
+                            .spec
+                            .arg_specs
+                            .iter()
+                            .all(|x| x.batchable && x.expandable)
+                    {
+                        "all_t".to_owned()
+                    } else {
+                        // print out out arg info, everything else in children
+                        // TODO: less terse?
+                        mn.spec
+                            .arg_specs
+                            .iter()
+                            .map(|arg_spec| {
+                                format!(
+                                    "{}{}",
+                                    TerseBool(arg_spec.batchable),
+                                    TerseBool(arg_spec.expandable)
+                                )
+                            })
+                            .collect::<Vec<_>>()
+                            .join(",")
+                    }
                 }
                 Circuit::Array(ac) => {
                     if self.bijection {
@@ -416,14 +452,16 @@ impl PrintOptions {
             None
         };
 
+        // ideally this would separate method, but to avoid breaking blame we keep like this
         fn recurse(
             circ: &Circuit,
             depth: usize,
             result: &mut String,
             seen_hashes: &mut HashMap<HashBytes, String>,
+            runnning_serial_number: &mut usize,
             selfy: &PrintOptions,
             is_last_child: &Vec<bool>,
-            term_at: Option<OpaqueIterativeMatcherVal>,
+            traversal: Option<OpaqueIterativeMatcherVal>,
             disallow_name_ident: Option<&HashMap<String, bool>>,
         ) -> Result<()> {
             if selfy.arrows {
@@ -457,8 +495,6 @@ impl PrintOptions {
                 ));
             }
 
-            let this_serial_number = seen_hashes.len().to_string();
-
             // allow_name_ident is_some iff !self.force_use_serial_numbers
             let disallow_name = disallow_name_ident
                 .zip(circ.name())
@@ -469,16 +505,22 @@ impl PrintOptions {
                 (_, Some(name)) => {
                     let escaped = selfy.string_escape(name);
                     let out = if disallow_name {
+                        let this_serial_number = runnning_serial_number.to_string();
+                        *runnning_serial_number += 1;
                         this_serial_number + " " + &escaped
                     } else {
                         escaped
                     };
                     (out.clone(), out)
                 }
-                (true, None) => (
-                    this_serial_number.clone() + " " + &circ.variant_string(),
-                    this_serial_number,
-                ),
+                (true, None) => {
+                    let this_serial_number = runnning_serial_number.to_string();
+                    *runnning_serial_number += 1;
+                    (
+                        this_serial_number.clone() + " " + &circ.variant_string(),
+                        this_serial_number,
+                    )
+                }
 
                 (false, None) => unreachable!(),
             };
@@ -486,55 +528,62 @@ impl PrintOptions {
             seen_hashes.insert(circ.info().hash, repeat_string);
 
             result.push_str(&line_prefix);
-            result.push_str(&selfy.repr_line_info(circ.clone().rc())?);
+            result.push_str(
+                &selfy
+                    .repr_line_info(circ.clone().rc())
+                    .with_context(|| format!("repr line info failed for circuit={:?}", circ))?,
+            );
             if selfy.colorer.is_some() {
                 result.push_str("\u{001b}[0m");
             }
 
-            let n_children = circ.children().count();
-            let new_term_at = if let Some(ta) = term_at {
-                let match_result = ta.opaque_match_iterate(circ.crc())?;
-                if match_result.finished {
-                    // term_at can't be used with bijection, so we can add ... without having to parse later
+            let num_children = circ.num_children();
+
+            let new_traversal_per =
+                if let Some(new_traversal_per) = op_traversal_per(traversal, circ.crc())? {
+                    new_traversal_per
+                } else {
+                    // all finished case
+                    // traversal can't be used with bijection, so we can add ... without having to parse later
                     if circ.children().count() != 0 {
                         result.push_str(" ...");
                     }
                     result.push('\n');
                     return Ok(());
-                }
-                Some(match_result.updated.unwrap_or(ta))
-            } else {
-                None
-            };
+                };
+
             result.push('\n');
-            for (i, child) in circ.children().enumerate() {
+            for (i, (child, new_traversal)) in circ.children().zip(new_traversal_per).enumerate() {
                 recurse(
                     &child,
                     depth + 1,
                     result,
                     seen_hashes,
+                    runnning_serial_number,
                     selfy,
                     &is_last_child
                         .iter()
                         .copied()
-                        .chain(std::iter::once(i == n_children - 1))
+                        .chain(std::iter::once(i == num_children - 1))
                         .collect(),
-                    new_term_at.clone(),
+                    new_traversal,
                     disallow_name_ident,
                 )?;
             }
             Ok(())
         }
         let mut result = String::new();
+        let mut runnning_serial_number = 0;
         for circuit in circuits {
             recurse(
                 &**circuit,
                 0,
                 &mut result,
                 &mut seen_hashes,
+                &mut runnning_serial_number,
                 &self,
                 &vec![],
-                self.term_at.clone(),
+                self.traversal.clone(),
                 disallow_name_ident.as_ref(),
             )?;
         }
@@ -560,7 +609,7 @@ impl PrintOptions {
     pub fn repr_circuits_depth(circuits: Vec<CircuitRc>, end_depth: usize) -> String {
         PrintOptions {
             bijection: false,
-            term_at: Some(OpaqueIterativeMatcherVal::for_end_depth(end_depth)),
+            traversal: Some(OpaqueIterativeMatcherVal::for_end_depth(end_depth)),
             ..Default::default()
         }
         .validate_ret()
@@ -570,32 +619,19 @@ impl PrintOptions {
     }
     #[staticmethod]
     #[pyo3(name = "print_depth")]
-    #[args(end_depth = "3")]
+    #[args(circuits = "*", end_depth = "3")]
     pub fn print_circuits_depth(circuits: Vec<CircuitRc>, end_depth: usize) {
         python_println!("{}", PrintOptions::repr_circuits_depth(circuits, end_depth))
     }
 
     #[staticmethod]
-    pub fn size_colorer() -> CircuitToColor {
-        CircuitToColor::new_dyn(Box::new(|circuit: CircuitRc| {
-            let size = circuit.info().numel_usize();
-            Ok(if size == usize::MAX {
-                3
-            } else if size > 10_000_000_000 {
-                4
-            } else if size > 300_000_000 {
-                1
-            } else if size > 300_000 {
-                2
-            } else {
-                0
-            })
-        }))
+    pub fn size_colorer() -> CircuitToColorCode {
+        CircuitToColorCode::new_dyn(Box::new(|c| Ok(circuit_to_size_color_code(c))))
     }
 
     #[staticmethod]
-    pub fn variant_colorer() -> CircuitToColor {
-        CircuitToColor::new_dyn(Box::new(|circuit: CircuitRc| {
+    pub fn variant_colorer() -> CircuitToColorCode {
+        CircuitToColorCode::new_dyn(Box::new(|circuit: CircuitRc| {
             let mut hasher = std::collections::hash_map::DefaultHasher::default();
             circuit.variant_string().hash(&mut hasher);
             Ok(hasher.finish() as usize)
@@ -603,27 +639,19 @@ impl PrintOptions {
     }
 
     #[staticmethod]
-    pub fn hash_colorer() -> CircuitToColor {
-        CircuitToColor::new_dyn(Box::new(|circuit: CircuitRc| {
+    pub fn hash_colorer() -> CircuitToColorCode {
+        CircuitToColorCode::new_dyn(Box::new(|circuit: CircuitRc| {
             Ok(circuit.info().hash_usize())
         }))
     }
     #[staticmethod]
-    pub fn fixed_color(color: usize) -> CircuitToColor {
-        CircuitToColor::new_dyn(Box::new(move |_circuit: CircuitRc| Ok(color)))
+    pub fn fixed_color(color: usize) -> CircuitToColorCode {
+        CircuitToColorCode::new_dyn(Box::new(move |_circuit: CircuitRc| Ok(color)))
     }
 
     #[staticmethod]
-    pub fn computability_colorer() -> CircuitToColor {
-        CircuitToColor::new_dyn(Box::new(|circuit: CircuitRc| {
-            Ok(if circuit.info().can_be_sampled {
-                1
-            } else if !circuit.info().is_explicitly_computable {
-                2
-            } else {
-                0
-            })
-        }))
+    pub fn computability_colorer() -> CircuitToColorCode {
+        CircuitToColorCode::new_dyn(Box::new(|c| Ok(circuit_to_computability_color_code(c))))
     }
 }
 
@@ -644,6 +672,31 @@ fn init_print_options() -> PrintOptions {
     PrintOptions::debug_default(end_depth)
 }
 
+pub fn circuit_to_size_color_code(circuit: CircuitRc) -> usize {
+    let size = circuit.info().numel_usize();
+    if size == usize::MAX {
+        3
+    } else if size > 10_000_000_000 {
+        4
+    } else if size > 300_000_000 {
+        1
+    } else if size > 300_000 {
+        2
+    } else {
+        0
+    }
+}
+
+pub fn circuit_to_computability_color_code(circuit: CircuitRc) -> usize {
+    if circuit.info().can_be_sampled {
+        1
+    } else if !circuit.info().is_explicitly_computable {
+        2
+    } else {
+        0
+    }
+}
+
 static DEBUG_PRINT_OPTIONS: GILLazy<Mutex<PrintOptions>> =
     GILLazy::new(|| Mutex::new(init_print_options()));
 
