@@ -13,6 +13,7 @@ use uuid::Uuid;
 use crate::{
     circuit_node_auto_impl, circuit_node_extra_impl,
     circuit_node_private::{CircuitNodeComputeInfoImpl, CircuitNodeHashItems},
+    circuit_utils::{child_name_with_maybe_paren, OperatorPriority},
     new_rc, new_rc_unwrap, CachedCircuitInfo, CircuitNode, CircuitNodeAutoName, CircuitRc,
     ConstructError, PyCircuitBase, Scalar, TensorEvalError,
 };
@@ -52,8 +53,12 @@ impl Tag {
 }
 
 impl CircuitNodeAutoName for Tag {
+    const PRIORITY: OperatorPriority = OperatorPriority::PostFix {};
+
     fn auto_name(&self, name: Option<String>) -> Option<String> {
-        name.or_else(|| self.node.name().map(|n| format!("Tag {}", n)))
+        name.or_else(|| {
+            child_name_with_maybe_paren(&Self::PRIORITY, self.node.clone()).map(|n| n + " tag")
+        })
     }
 }
 
@@ -176,17 +181,12 @@ impl DiscreteVar {
 }
 
 impl CircuitNodeAutoName for DiscreteVar {
+    const PRIORITY: OperatorPriority = OperatorPriority::PostFix {};
+
     fn auto_name(&self, name: Option<String>) -> Option<String> {
+        // Ignores probs and groups because we almost never care about the probs and groups name
         name.or_else(|| {
-            if self.probs_and_group.name().is_none() || self.values.name().is_none() {
-                None
-            } else {
-                Some(format!(
-                    "DiscreteVar {} {}",
-                    self.values.name().unwrap(),
-                    self.probs_and_group.name().unwrap()
-                ))
-            }
+            child_name_with_maybe_paren(&Self::PRIORITY, self.values.clone()).map(|n| n + " var")
         })
     }
 }
@@ -348,13 +348,16 @@ impl CircuitNodeAutoName for StoredCumulantVar {
                 None
             } else {
                 Some(
-                    "StoredCumulantVar ".to_owned()
+                    "StoredCumulantVar(".to_owned()
                         + &self
                             .cumulants
-                            .values()
-                            .map(|x| Self::shorten_child_name(x.name().unwrap()))
+                            .iter()
+                            .map(|(r, x)| {
+                                format!("{r}: {}", Self::shorten_child_name(x.name().unwrap()))
+                            })
                             .collect::<Vec<String>>()
-                            .join(", "),
+                            .join(", ")
+                        + ")",
                 )
             }
         })
@@ -445,6 +448,7 @@ impl StoredCumulantVar {
 
     #[staticmethod]
     #[pyo3(name = "new_mv")]
+    #[args(higher_cumulants = "Default::default()", uuid = "None", name = "None")]
     pub fn new_mv_py(
         mean: CircuitRc,
         variance: CircuitRc,
