@@ -1953,20 +1953,20 @@ pub fn einsum_permute_to_rearrange(einsum: &Einsum) -> Option<Rearrange> {
 #[base_exception(PyValueError)]
 #[derive(Error, Debug, Clone)]
 pub enum PushDownIndexError {
-    #[error("index_node={index_node:?} index_non_axis={index_non_axis} index={index}")]
+    #[error("index_node={index_node:?} index_non_axis={index_non_axis} index={index} ({e_name})")]
     NoopOnConcat {
         index_node: Index,
         index_non_axis: TensorIndex,
         index: TensorIndex,
     },
-    #[error("index_node={index_node:?} is_batchable={is_batchable} num_non_batch_out={num_non_batch_out} inner_rank={inner_rank}")]
+    #[error("index_node={index_node:?} is_batchable={is_batchable} num_non_batch_out={num_non_batch_out} inner_rank={inner_rank} ({e_name})")]
     NoopOnGeneralFunction {
         index_node: Index,
         is_batchable: bool,
         num_non_batch_out: usize,
         inner_rank: usize,
     },
-    #[error("index_node={index_node:?} is_batchable={is_batchable} num_non_batch_out={num_non_batch_out} inner_rank={inner_rank} top_axis_indices={top_axis_indices:?}")]
+    #[error("index_node={index_node:?} is_batchable={is_batchable} num_non_batch_out={num_non_batch_out} inner_rank={inner_rank} top_axis_indices={top_axis_indices:?} ({e_name})")]
     GeneralFunctionSomeAxesNotPossible {
         index_node: Index,
         is_batchable: bool,
@@ -1975,40 +1975,40 @@ pub enum PushDownIndexError {
         top_axis_indices: Vec<TensorAxisIndex>,
     },
     // TODO: improve errors as needed
-    #[error("index_node={index_node:?}")]
+    #[error("index_node={index_node:?} ({e_name})")]
     EinsumNoop { index_node: Index },
-    #[error("index_node={index_node:?} out_index={out_index:?} out_axes={out_axes:?} int_indices={int_indices:?}")]
+    #[error("index_node={index_node:?} out_index={out_index:?} out_axes={out_axes:?} int_indices={int_indices:?} ({e_name})")]
     EinsumSomeAxesNotPossible {
         index_node: Index,
         out_index: TensorIndex,
         out_axes: EinsumAxes,
         int_indices: HashMap<u8, Option<TensorAxisIndex>>,
     },
-    #[error("index_node={index_node:?} spec={spec:?}")]
+    #[error("index_node={index_node:?} spec={spec:?} ({e_name})")]
     RearrangeNotPossible {
         index_node: Index,
         spec: RearrangeSpec,
     },
-    #[error("index_node={index_node:?}")]
+    #[error("index_node={index_node:?} ({e_name})")]
     ScatterNoop { index_node: Index },
-    #[error("index_node={index_node:?} out_index={out_index:?}")]
+    #[error("index_node={index_node:?} out_index={out_index:?} ({e_name})")]
     ScatterSomeAxesNotPossible {
         index_node: Index,
         out_index: TensorIndex,
         // TODO: more
     },
-    #[error("can't push down index through other index, but you can use 'index_fuse'\nindex_node={index_node:?} circ_type={circ_type:?}")]
+    #[error("can't push down index through other index, but you can use 'index_fuse'\nindex_node={index_node:?} circ_type={circ_type:?} ({e_name})")]
     ThroughIndex {
         index_node: Index,
         circ_type: CircuitType,
     },
-    #[error("index_node={index_node:?} circ_type={circ_type:?}")]
+    #[error("index_node={index_node:?} circ_type={circ_type:?} ({e_name})")]
     UnimplementedType {
         index_node: Index,
         circ_type: CircuitType,
     },
 
-    #[error("tensors aren't currently supported tensor={tensor:?}")]
+    #[error("tensors aren't currently supported tensor={tensor:?} ({e_name})")]
     ConcatSplitSectionsTensorUnsupported { tensor: IndexTensor },
 }
 
@@ -2017,14 +2017,14 @@ pub enum PushDownIndexError {
 #[base_exception(PyValueError)]
 #[derive(Error, Debug, Clone)]
 pub enum DistributeError {
-    #[error("add={add:?} do_broadcasts={do_broadcasts}")]
+    #[error("add={add:?} do_broadcasts={do_broadcasts} ({e_name})")]
     CouldntBroadcast { add: Add, do_broadcasts: bool },
-    #[error("einsum={einsum:?} operand_idx={operand_idx}")]
+    #[error("einsum={einsum:?} operand_idx={operand_idx} ({e_name})")]
     OperandIsNotAdd { einsum: Einsum, operand_idx: usize },
-    #[error("einsum={einsum:?} operand_idx={operand_idx}")]
+    #[error("einsum={einsum:?} operand_idx={operand_idx} ({e_name})")]
     OperandIdxTooLarge { einsum: Einsum, operand_idx: usize },
-    #[error("einsum={einsum:?} operand_idx={operand_idx}")]
+    #[error("einsum={einsum:?} operand_idx={operand_idx} ({e_name})")]
     EmptyAddUnsupported { einsum: Einsum, operand_idx: usize },
-    #[error("einsum={einsum:?} operand_idx={operand_idx}")]
+    #[error("einsum={einsum:?} operand_idx={operand_idx} ({e_name})")]
     Noop { einsum: Einsum, operand_idx: usize },
 }
