@@ -1,3 +1,4 @@
+#![feature(let_chains)]
 #![feature(map_try_insert)]
 pub mod algebraic_rewrite;
 pub mod batching;
@@ -17,5 +18,6 @@ pub mod sampling;
 pub mod scatter_rewrite;
 pub mod schedule_send;
 pub mod scheduled_execution;
+pub mod scheduling_alg;
 pub mod scheduling_z3;
 pub mod server;
