@@ -287,14 +287,14 @@ pub struct ScheduleStats {
 #[base_exception(PyValueError)]
 #[derive(Error, Clone, Debug)]
 pub enum SchedulingOOMError {
-    #[error("{string}")]
+    #[error("{string} ({e_name})")]
     Many {
         max_memory: usize,
         memory_chunks: usize,
         node_memories: Vec<usize>,
         string: String,
     },
-    #[error("Single element doesn't fit {numel}")]
+    #[error("Single element doesn't fit {numel} ({e_name})")]
     Single { numel: BigUint },
 }
 
@@ -303,7 +303,7 @@ pub enum SchedulingOOMError {
 #[base_exception(PyValueError)]
 #[derive(Error, Debug)]
 pub enum SchedulingError {
-    #[error("circuit={circuit:?}")]
+    #[error("circuit={circuit:?} ({e_name})")]
     NotExplicitlyComputable { circuit: CircuitRc },
 }
 
