@@ -424,18 +424,18 @@ impl Matcher {
         MatcherData::Any(matchers).into()
     }
 
-    #[pyo3(name = "mk_not")]
+    #[pyo3(name = "new_not")]
     pub fn not(&self) -> Self {
         MatcherData::Not(self.crc()).into()
     }
 
     #[args(others = "*")]
-    pub fn mk_and(&self, others: Vec<MatcherRc>) -> Self {
+    pub fn new_and(&self, others: Vec<MatcherRc>) -> Self {
         Self::all([self.crc()].into_iter().chain(others).collect())
     }
 
     #[args(others = "*")]
-    pub fn mk_or(&self, others: Vec<MatcherRc>) -> Self {
+    pub fn new_or(&self, others: Vec<MatcherRc>) -> Self {
         Self::any([self.crc()].into_iter().chain(others).collect())
     }
 
