@@ -25,7 +25,10 @@ use rr_util::{
 use smallvec::SmallVec as Sv;
 use thiserror::Error;
 
-use crate::{Batcher, IterateMatchResults, IterativeMatcherRc, Matcher, Transform, TransformRc};
+use crate::{
+    iterative_matcher::function_per_child, Batcher, IterateMatchResults, IterativeMatcherRc,
+    Matcher, Transform, TransformRc,
+};
 
 fn count_elements(s: &Shape) -> usize {
     s.iter().product()
@@ -560,17 +563,11 @@ impl SampleTransform {
         circuit: CircuitRc,
         matcher: IterativeMatcherRc,
     ) -> Result<CircuitRc> {
-        let IterateMatchResults {
-            updated,
-            finished,
-            found,
-        } = matcher.match_iterate(circuit.clone())?;
-        let mut new_circuit = circuit.clone();
-
-        if !finished {
-            let new_matcher = updated.unwrap_or(matcher.clone());
-            new_circuit = circuit.map_children(|c| self_.estimate_impl(c, new_matcher.clone()))?;
-        }
+        let IterateMatchResults { updated, found } = matcher.match_iterate(circuit.clone())?;
+        let mut new_circuit =
+            function_per_child(updated, matcher.clone(), circuit, |c, matcher| {
+                self_.estimate_impl(c, matcher)
+            })?;
         if found && let Some(cum) = new_circuit.as_cumulant() {
             new_circuit = self_.estimate_cumulant(cum, matcher)?;
         }
@@ -651,11 +648,11 @@ impl SampleTransform {
 #[base_exception(PyValueError)]
 #[derive(Error, Debug, Clone)]
 pub enum SampleError {
-    #[error("This var wasn't handled by the corresponding function.")]
+    #[error("This var wasn't handled by the corresponding function. ({e_name})")]
     UnhandledVarError { circ: CircuitRc },
-    #[error("some probs_and_group has ndim != 1: {groups:?}")]
+    #[error("some probs_and_group has ndim != 1: {groups:?} ({e_name})")]
     GroupWithIncorrectNdim { groups: Vec<CircuitRc> },
-    #[error("passed in {subset_len} but {groups_len} groups")]
+    #[error("passed in {subset_len} but {groups_len} groups ({e_name})")]
     DifferentNumSubsetsThanGroups {
         subset_len: usize,
         groups_len: usize,
