@@ -477,13 +477,26 @@ pub fn use_rust_comp<T: PartialOrd>(l: &T, r: &T, comp_op: CompareOp) -> bool {
     }
 }
 
+/// macro instead of generic type because of pyo3 bs
 #[macro_export]
 macro_rules! make_single_many {
+    ($name:ident, $type:ty, $container:ident, into_py) => {
+        make_single_many!($name, $type, $container);
+
+        impl IntoPy<PyObject> for $name {
+            fn into_py(self, py: Python<'_>) -> PyObject {
+                match self {
+                    Self::Many(x) => x.into_py(py),
+                    Self::Single(x) => x.into_py(py),
+                }
+            }
+        }
+    };
     ($name:ident, $type:ty, $container:ident) => {
-        #[derive(Clone, FromPyObject)]
+        #[derive(Clone, Debug, FromPyObject)]
         pub enum $name {
-            Single($type),
             Many($container<$type>),
+            Single($type),
         }
 
         impl $name {
@@ -494,6 +507,20 @@ macro_rules! make_single_many {
                 }
             }
 
+            pub fn is_many(&self) -> bool {
+                matches!(self, Self::Many(_))
+            }
+            pub fn is_single(&self) -> bool {
+                matches!(self, Self::Single(_))
+            }
+
+            pub fn map<F: FnMut($type) -> $type>(self, mut f: F) -> Self {
+                match self {
+                    Self::Single(x) => Self::Single(f(x)),
+                    Self::Many(x) => Self::Many(x.into_iter().map(f).collect()),
+                }
+            }
+
             pub fn len(&self) -> usize {
                 match self {
                     Self::Single(_) => 1,
