@@ -1375,39 +1375,41 @@ impl RearrangeSpec {
 #[base_exception(PyValueError)]
 #[derive(Error, Debug, Clone)]
 pub enum RearrangeSpecError {
-    #[error("RearrangeSpec not conformable, {spec:?} {shape:?}")]
+    #[error("RearrangeSpec not conformable, {spec:?} {shape:?} ({e_name})")]
     InputNotConformable { spec: RearrangeSpec, shape: Shape },
 
-    #[error("RearrangeSpec has wildcard sizes, {spec:?}=")]
+    #[error("RearrangeSpec has wildcard sizes, {spec:?} ({e_name})")]
     HasWildcardSizes { spec: RearrangeSpec },
 
-    #[error("RearrangeSpec cannot be converted to no-(), no-squeeze format, {spec:?}=")]
+    #[error("RearrangeSpec cannot be converted to no-(), no-squeeze format, {spec:?} ({e_name})")]
     NotConvertable { spec: RearrangeSpec },
 
-    #[error("RearrangeSpec has too many wildcard sizes, {spec:?}")]
+    #[error("RearrangeSpec has too many wildcard sizes, {spec:?} ({e_name})")]
     TooManyWildcardSizes { spec: RearrangeSpec },
 
-    #[error("counts.len()={counts_len} != axes.len()={axes_len}")]
+    #[error("counts.len()={counts_len} != axes.len()={axes_len} ({e_name})")]
     AxesAndCountsDontMatch { axes_len: usize, counts_len: usize },
 
-    #[error("Rearranges can't be fused, {inner:?}\n{outer:?}")]
+    #[error("Rearranges can't be fused, {inner:?}\n{outer:?} ({e_name})")]
     NotFusable {
         inner: RearrangeSpec,
         outer: RearrangeSpec,
     },
 
-    #[error("Repeated axes={axes:?} unique_count={unique_count} actual_count={actual_count}")]
+    #[error(
+        "Repeated axes={axes:?} unique_count={unique_count} actual_count={actual_count} ({e_name})"
+    )]
     RepeatedAxes {
         axes: Vec<usize>,
         unique_count: usize,
         actual_count: usize,
     },
 
-    #[error("ints not unique is_input={is_input}, ints={ints:?}")]
+    #[error("ints not unique is_input={is_input}, ints={ints:?} ({e_name})")]
     IntsNotUnique { is_input: bool, ints: Vec<AxisInt> },
 
     #[error(
-        "inp int not in output difference={difference:?}, ints_in_inp={ints_in_inp:?}, ints_in_out={ints_in_out:?}"
+        "inp int not in output difference={difference:?}, ints_in_inp={ints_in_inp:?}, ints_in_out={ints_in_out:?} ({e_name})"
     )]
     InpNotInOut {
         difference: HashSet<AxisInt>,
@@ -1415,10 +1417,10 @@ pub enum RearrangeSpecError {
         ints_in_out: HashSet<AxisInt>,
     },
 
-    #[error("int not in sizes vec, len_sizes={len_sizes}, max_int={max_int}")]
+    #[error("int not in sizes vec, len_sizes={len_sizes}, max_int={max_int} ({e_name})")]
     IntNotInSizes { max_int: u8, len_sizes: usize },
 
-    #[error("int which is only in output doesn't have size, only_in_output={only_in_output:?}, corresponding_sizes={corresponding_sizes:?}, int_sizes={int_sizes:?}")]
+    #[error("int which is only in output doesn't have size, only_in_output={only_in_output:?}, corresponding_sizes={corresponding_sizes:?}, int_sizes={int_sizes:?} ({e_name})")]
     IntOnlyInOutputWithoutSize {
         only_in_output: Vec<AxisInt>,
         corresponding_sizes: Vec<OpSize>,
@@ -1426,17 +1428,17 @@ pub enum RearrangeSpecError {
     },
 
     // TODO: improve error message to highlight failure
-    #[error("ints={ints:?} int_sizes={int_sizes:?}")]
+    #[error("ints={ints:?} int_sizes={int_sizes:?} ({e_name})")]
     InputAxisHasMultipleWildCards { ints: RInts, int_sizes: OpShape },
 
     // TODO: improve error message to highlight failure
-    #[error("len_shape={len_shape} > 255")]
+    #[error("len_shape={len_shape} > 255 ({e_name})")]
     LenShapeTooLarge { len_shape: usize },
 
-    #[error("axes_to_combine={axes_to_combine:?} for range 0..{n}")]
+    #[error("axes_to_combine={axes_to_combine:?} for range 0..{n} ({e_name})")]
     AxesToCombineNotSubset { axes_to_combine: RInnerInts, n: u8 },
 
-    #[error("Tried to unflatten with ndim=0")]
+    #[error("Tried to unflatten with ndim=0 ({e_name})")]
     CantUnflattenScalar {},
 }
 
@@ -1446,10 +1448,10 @@ pub enum RearrangeSpecError {
 #[derive(Error, Debug, Clone)]
 pub enum PermError {
     // TODO: improve errors
-    #[error("ints={ints:?}")]
+    #[error("ints={ints:?} ({e_name})")]
     IntsNotUnique { ints: Vec<usize> },
 
-    #[error("ints={ints:?} count={count}")]
+    #[error("ints={ints:?} count={count} ({e_name})")]
     NotContiguousInts { ints: Vec<usize>, count: usize },
 }
 
@@ -1459,10 +1461,10 @@ pub enum PermError {
 #[derive(Error, Debug, Clone)]
 pub enum RearrangeParseError {
     // TODO: improve errors
-    #[error("string={string:?}")]
+    #[error("string={string:?} ({e_name})")]
     ArrowIssue { string: String },
-    #[error("string={string:?}")]
+    #[error("string={string:?} ({e_name})")]
     FailedToMatchRegex { string: String },
-    #[error("string={string:?}")]
+    #[error("string={string:?} ({e_name})")]
     TooManyAxes { string: String },
 }
