@@ -30,8 +30,11 @@ pub fn symbolic_sizes() -> Vec<usize> {
 }
 
 #[derive(Clone, PartialEq, Eq, Hash)]
+#[pyclass]
 pub struct SymbolicSizeProduct {
+    #[pyo3(get)]
     pub other_factor: usize,
+    #[pyo3(get)]
     pub symbolic_sizes: Vec<usize>,
 }
 
@@ -188,9 +191,12 @@ impl SymbolicSizeProduct {
 #[derive(Clone, Debug, Eq, PartialEq, Hash)]
 pub struct SymbolicSizeConstraints(Vec<SymbolicSizeConstraint>);
 
+#[pyclass]
 #[derive(Clone, Eq, PartialEq, Hash)]
 pub struct SymbolicSizeConstraint {
+    #[pyo3(get)]
     pub l: SymbolicSizeProduct,
+    #[pyo3(get)]
     pub r: SymbolicSizeProduct,
 }
 
@@ -348,12 +354,12 @@ fn check_constraints(constraints: Vec<SymbolicSizeConstraint>) -> z3::SatResult
 #[base_exception(PyOverflowError)]
 #[derive(Error, Debug, Clone)]
 pub enum SymbolicSizeOverflowError {
-    #[error("symbolic_size_product={symbolic_size_product:?}")]
+    #[error("symbolic_size_product={symbolic_size_product:?} ({e_name})")]
     ProductTooLarge {
         symbolic_size_product: SymbolicSizeProduct,
     },
 
-    #[error("i={i} >= {}", SYMBOLIC_SIZES.len())]
+    #[error("i={i} >= {} ({e_name})", SYMBOLIC_SIZES.len())]
     SymbolicSizeNumberOutOfBounds { i: usize },
 }
 
@@ -362,17 +368,17 @@ pub enum SymbolicSizeOverflowError {
 #[base_exception(PyOverflowError)]
 #[derive(Error, Debug, Clone)]
 pub enum SymbolicSizeSetError {
-    #[error("size_prod={size_prod} is not 0, but tried to set to 0")]
+    #[error("size_prod={size_prod} is not 0, but tried to set to 0 ({e_name})")]
     TriedToSetNonZeroToZero { size_prod: SymbolicSizeProduct },
 
-    #[error("factor={factor} doesn't divide set_to={set_to} (trying to set symbolic size). size_prod={size_prod}")]
+    #[error("factor={factor} doesn't divide set_to={set_to} (trying to set symbolic size). size_prod={size_prod} ({e_name})")]
     FactorDoesntDivideSetTo {
         factor: usize,
         set_to: usize,
         size_prod: SymbolicSizeProduct,
     },
 
-    #[error("l_factor={l_factor} != r_factor={r_factor}\nl_size_prod={l_size_prod} r_size_prod={r_size_prod}")]
+    #[error("l_factor={l_factor} != r_factor={r_factor}\nl_size_prod={l_size_prod} r_size_prod={r_size_prod} ({e_name})")]
     NoSymbolicAndSizesNotEqual {
         l_factor: usize,
         r_factor: usize,
@@ -380,7 +386,7 @@ pub enum SymbolicSizeSetError {
         r_size_prod: SymbolicSizeProduct,
     },
 
-    #[error("constraints={constraints:?}")]
+    #[error("constraints={constraints:?} ({e_name})")]
     FailedToSatisfyContraints {
         constraints: Vec<SymbolicSizeConstraint>,
     },
