@@ -68,19 +68,19 @@ pub fn broadcast_shapes(shapes: &Vec<Shape>) -> Result<Shape> {
 #[base_exception(PyValueError)]
 #[derive(Error, Debug, Clone)]
 pub enum MiscInputError {
-    #[error("Shapes aren't broadcastable, {shapes:?}")]
+    #[error("Shapes aren't broadcastable, {shapes:?} ({e_name})")]
     NotBroadcastable { shapes: Vec<Shape> },
 
-    #[error("item={item}, count={count} (needed -{count} <= {item} < {count})")]
+    #[error("item={item}, count={count} (needed -{count} <= {item} < {count}) ({e_name})")]
     ItemOutOfBounds { item: i64, count: usize },
 
-    #[error("Children multiple dtypes {a:?} {b:?}")]
+    #[error("Children multiple dtypes {a:?} {b:?} ({e_name})")]
     ChildrenMultipleDtypes {
         a: Option<String>,
         b: Option<String>,
     },
 
-    #[error("Children multiple dtypes {a:?} {b:?}")]
+    #[error("Children multiple dtypes {a:?} {b:?} ({e_name})")]
     ChildrenMultipleDevices {
         a: Option<String>,
         b: Option<String>,
@@ -92,10 +92,10 @@ pub enum MiscInputError {
 #[base_exception(PyValueError)]
 #[derive(Error, Debug, Clone)]
 pub enum IndexError {
-    #[error("Only ndim of 1 is supported for index tensors and got {ndim} (tensor: {tensor:?})")]
+    #[error("Only ndim of 1 is supported for index tensors and got {ndim} (tensor: {tensor:?}) ({e_name})")]
     TensorNDimNot1 { ndim: usize, tensor: Tensor },
 
-    #[error("Index {axis:?} out of bounds, index: {index:?} shape: {shape:?}. NB: Rust circuit slices don't saturate like Python ones do.")]
+    #[error("Index {axis:?} out of bounds, index: {index:?} shape: {shape:?}. NB: Rust circuit slices don't saturate like Python ones do. ({e_name})")]
     IndexOutOfBounds {
         index: TensorIndex,
         shape: Shape,
@@ -104,10 +104,10 @@ pub enum IndexError {
         l: usize,
     },
 
-    #[error("index rank too high: {index_rank} vs {node_rank}")]
+    #[error("index rank too high: {index_rank} vs {node_rank} ({e_name})")]
     IndexRankTooHigh { index_rank: usize, node_rank: usize },
 
-    #[error("slice={slice:?}")]
+    #[error("slice={slice:?} ({e_name})")]
     UnsupportedSlice { slice: Slice },
 }
 
@@ -988,11 +988,14 @@ const REGEX_101_MSG: &str = "\nTry using regex101.com : )\n(TODO: auto generate
 #[base_exception(PyValueError)]
 #[derive(Error, Debug, Clone)]
 pub enum ParseError {
-    #[error("regex didn't match '{line}'\nregex='{regex_string}'{}", REGEX_101_MSG)]
+    #[error(
+        "regex didn't match '{line}'\nregex='{regex_string}'{}\n({e_name})",
+        REGEX_101_MSG
+    )]
     RegexDidntMatch { line: String, regex_string: String },
 
     #[error(
-        "regex didn't match group={group} for '{line}'\n{}\nregex='{regex_string}'{}",
+        "regex didn't match group={group} for '{line}'\n{}\nregex='{regex_string}'{}\n({e_name})",
         "Note: this error can also be caused by failing to lookup a reference circuit",
         REGEX_101_MSG
     )]
@@ -1002,29 +1005,29 @@ pub enum ParseError {
         group: usize,
     },
 
-    #[error("Parsing number failed err={err_string}")]
+    #[error("Parsing number failed err={err_string} ({e_name})")]
     NumberFail { err_string: String },
 
-    #[error("extra='{extra}'")]
+    #[error("extra='{extra}' ({e_name})")]
     ExpectedNoExtraInfo { extra: String },
 
-    #[error("Parsing failed {string}")]
+    #[error("Parsing failed {string} ({e_name})")]
     Fail { string: String },
 
-    #[error("Expected UUID string, found  '{string}'\nerr_msg={err_msg}")]
+    #[error("Expected UUID string, found  '{string}'\nerr_msg={err_msg} ({e_name})")]
     InvalidUuid { string: String, err_msg: String },
 
-    #[error("Parsing wrong number of children, expected {expected} found {found}, ident={ident}")]
+    #[error("Parsing wrong number of children, expected {expected} found {found}, ident={ident} ({e_name})")]
     WrongNumberChildren {
         expected: usize,
         found: usize,
         ident: String,
     },
 
-    #[error("effectively negative indent!\n{}\n{}\n{}",
+    #[error("effectively negative indent!\n{}\n{}\n{}\n({e_name})",
         format!("this_num_spaces_base={} < first_num_spaces={}", this_num_spaces_base, first_num_spaces),
         format!("first_line='{}'", first_line),
-        format!("this_line='{}'", this_line),
+        format!("this_line='{}'", this_line)
         )]
     LessIndentationThanFirstItem {
         this_num_spaces_base: usize,
@@ -1033,64 +1036,64 @@ pub enum ParseError {
         this_line: String,
     },
 
-    #[error("Parsing invalid indentation, tab width {tab_width} num spaces {spaces} stack indentation {stack_indentation}")]
+    #[error("Parsing invalid indentation, tab width {tab_width} num spaces {spaces} stack indentation {stack_indentation} ({e_name})")]
     InvalidIndentation {
         spaces: usize,
         tab_width: usize,
         stack_indentation: usize,
     },
 
-    #[error("Parsing invalid circuit variant {v}")]
+    #[error("Parsing invalid circuit variant {v} ({e_name})")]
     InvalidVariant { v: String },
 
-    #[error("Parsing invalid serial number {serial_number}")]
+    #[error("Parsing invalid serial number {serial_number} ({e_name})")]
     InvalidSerialNumber { serial_number: usize },
 
-    #[error("Parsing Extra Unneeded String '{string}'")]
+    #[error("Parsing Extra Unneeded String '{string}' ({e_name})")]
     ExtraUnneededString { string: String },
 
-    #[error("Parsing shape needed but not provided on {variant}")]
+    #[error("Parsing shape needed but not provided on {variant} ({e_name})")]
     ShapeNeeded { variant: String },
 
-    #[error("Circuit by name not found: {name}")]
+    #[error("Circuit by name not found: {name} ({e_name})")]
     CircuitRefNotFound { name: String },
 
-    #[error("Einsum string invalid {string} {substring}")]
+    #[error("Einsum string invalid {string} {substring} ({e_name})")]
     EinsumStringInvalid { string: String, substring: String },
 
-    #[error("Einsum string doesn't have arrow {string}")]
+    #[error("Einsum string doesn't have arrow {string} ({e_name})")]
     EinsumStringNoArrow { string: String },
 
-    #[error("actual_num_circuits={actual_num_circuits}")]
+    #[error("actual_num_circuits={actual_num_circuits} ({e_name})")]
     ExpectedOneCircuitGotMultiple { actual_num_circuits: usize },
 
-    #[error("expected 't' or 'f' got='{got}'")]
+    #[error("expected 't' or 'f' got='{got}' ({e_name})")]
     InvalidTerseBool { got: String },
 
-    #[error("reference_name={reference_name}")]
+    #[error("reference_name={reference_name} ({e_name})")]
     ReferenceCircuitHasChildrenInParseStr { reference_name: String },
 
-    #[error("reference_name={reference_name}")]
+    #[error("reference_name={reference_name} ({e_name})")]
     ReferenceCircuitNameFollowedByAdditionalInfo {
         reference_name: String,
         line: String,
     },
 
-    #[error("repeat_ident={repeat_ident} \n{err_strs}line='{line}'")]
+    #[error("repeat_ident={repeat_ident} \n{err_strs}line='{line}' ({e_name})")]
     OnCircuitRepeatInfoIsNotSame {
         repeat_ident: String,
         line: String,
         err_strs: String,
     },
 
-    #[error("num_chars={num_chars} extra='{extra}' name={name:?}\nmodule extra should look like 'tt,tf,ft' and we check for 2 chars per comma delimited item.")]
+    #[error("num_chars={num_chars} extra='{extra}' name={name:?}\nmodule extra should look like 'tt,tf,ft' and we check for 2 chars per comma delimited item.\n({e_name})")]
     Expected2CharsPerForModuleExtra {
         name: Option<String>,
         num_chars: usize,
         extra: String,
     },
 
-    #[error("num_children={num_children} != 2 * num_args={num_args:?} + 1\nname={name:?} extra={extra}\n{}\n{}\n{}",
+    #[error("num_children={num_children} != 2 * num_args={num_args:?} + 1\nname={name:?} extra={extra}\n{}\n{}\n{}\n({e_name})",
     "modules have 1 child for the spec and then a symbol and a input circuit for each arg (2 * num_args + 1)",
     "this implies the number of children is always odd",
     "num_args is based on parsing the batchable and expandable info from 'extra' (see above) (in a form like 'tt,tf,ft' (Some(3)) or 'all_t' (None))")]
@@ -1101,10 +1104,10 @@ pub enum ParseError {
         num_args: Option<usize>,
     },
 
-    #[error("factors={factors:?} string='{string}'")]
+    #[error("factors={factors:?} string='{string}' ({e_name})")]
     FactorProductTooLarge { factors: Vec<usize>, string: String },
 
-    #[error("i={i} >= bound={bound}")]
+    #[error("i={i} >= bound={bound} ({e_name})")]
     SymbolicSizeNumberOutOfBounds { i: usize, bound: usize },
 }
 
