@@ -9,12 +9,9 @@ use circuit_rewrites::{
     deep_rewrite::SimpConfigError,
     scheduled_execution::{SchedulingError, SchedulingOOMError},
 };
-use get_update_node::sample_transform::SampleError;
-use nb_operations::nest::NestError;
-use pyo3::{
-    types::{PyModule, PyType},
-    Py, PyResult, Python,
-};
+use get_update_node::{iterative_matcher::IterativeMatcherError, sample_transform::SampleError};
+use nb_operations::{modules::ModuleBindError, nest::NestError};
+use pyo3::{types::PyModule, PyErr, PyResult, Python};
 use rr_util::{
     errors_util::HasPythonException,
     rearrange_spec::{PermError, RearrangeParseError, RearrangeSpecError},
@@ -23,16 +20,13 @@ use rr_util::{
 };
 macro_rules! setup_errors {
     ($($t:ty),* $(,)?) => {
-        pub fn get_exception(err: &anyhow::Error) -> Option<Py<PyType>> {
-            if let Some(x) = pyo3::anyhow::get_exception_from_base_error(err) {
-                return Some(x);
-            }
+        pub fn anyhow_to_py_err(err: anyhow::Error) -> PyErr {
             $(
                 if let Some(x) = err.root_cause().downcast_ref::<$t>() {
-                    return Some(x.get_python_exception_type());
+                    return x.get_py_err(format!("{:?}", err));
                 }
             )*
-            None
+            pyo3::anyhow::default_anyhow_to_py_err(err)
         }
         pub fn register_exceptions(py: Python<'_>, m: &PyModule) -> PyResult<()> {
             $(
@@ -74,4 +68,6 @@ setup_errors!(
     ParseArgError,
     SymbolicSizeOverflowError,
     SymbolicSizeSetError,
+    IterativeMatcherError,
+    ModuleBindError
 );
