@@ -15,7 +15,7 @@ use pyo3::{types::PyModule, wrap_pyfunction, PyResult, Python};
 pub mod error;
 #[pyo3::pymodule]
 fn rust_circuit(py: Python<'_>, m: &PyModule) -> PyResult<()> {
-    pyo3::anyhow::set_get_exception(std::sync::Arc::new(error::get_exception));
+    pyo3::anyhow::set_anyhow_to_py_err(Box::new(error::anyhow_to_py_err));
 
     use circuit_base::{
         circuit_utils::{
@@ -112,10 +112,19 @@ fn rust_circuit(py: Python<'_>, m: &PyModule) -> PyResult<()> {
     m.add_class::<circuit_base::DiscreteVar>()?;
     m.add_class::<circuit_base::StoredCumulantVar>()?;
     m.add_class::<circuit_base::Cumulant>()?;
+    m.add_class::<circuit_base::SetSymbolicShape>()?;
+    m.add_class::<rr_util::symbolic_size::SymbolicSizeProduct>()?;
+    m.add_class::<rr_util::symbolic_size::SymbolicSizeConstraint>()?;
     m.add_class::<get_update_node::Matcher>()?;
     m.add_class::<get_update_node::matcher::RegexWrap>()?;
     m.add_class::<get_update_node::IterativeMatcher>()?;
+    m.add(
+        "traversal",
+        get_update_node::IterativeMatcher::noop_traversal(),
+    )?;
     m.add_class::<get_update_node::IterateMatchResults>()?;
+    m.add_class::<circuit_base::opaque_iterative_matcher::Finished>()?;
+    m.add("FINISHED", circuit_base::opaque_iterative_matcher::Finished)?;
     m.add_class::<get_update_node::Transform>()?;
     m.add_class::<get_update_node::Updater>()?;
     m.add_class::<get_update_node::BoundUpdater>()?;
@@ -123,6 +132,7 @@ fn rust_circuit(py: Python<'_>, m: &PyModule) -> PyResult<()> {
     m.add_class::<get_update_node::BoundGetter>()?;
     m.add_class::<get_update_node::Batcher>()?;
     m.add_class::<circuit_base::print::PrintOptions>()?;
+    m.add_class::<circuit_base::print_html::PrintHtmlOptions>()?;
     m.add_function(wrap_pyfunction!(
         circuit_base::print::set_debug_print_options,
         m
@@ -307,6 +317,7 @@ fn rust_circuit(py: Python<'_>, m: &PyModule) -> PyResult<()> {
     m.add_function(wrap_pyfunction!(circuit_base::deep_map_py, m)?)?;
 
     m.add_function(wrap_pyfunction!(circuit_base::visit_circuit_py, m)?)?;
+    m.add_function(wrap_pyfunction!(circuit_base::all_children, m)?)?;
 
     m.add_function(wrap_pyfunction!(filter_nodes_py, m)?)?;
     m.add_function(wrap_pyfunction!(replace_nodes_py, m)?)?;
@@ -336,6 +347,10 @@ fn rust_circuit(py: Python<'_>, m: &PyModule) -> PyResult<()> {
         circuit_base::module::inline_all_modules,
         m
     )?)?;
+    m.add_function(wrap_pyfunction!(
+        circuit_base::module::conform_all_modules,
+        m
+    )?)?;
     m.add_function(wrap_pyfunction!(
         circuit_base::expand_node::replace_expand_bottom_up_dict_py,
         m
@@ -415,6 +430,11 @@ fn rust_circuit(py: Python<'_>, m: &PyModule) -> PyResult<()> {
         nb_operations::distribute_and_factor::distribute,
         m
     )?)?;
+    m.add_function(wrap_pyfunction!(
+        nb_operations::modules::module_new_bind,
+        m
+    )?)?;
+    m.add_class::<nb_operations::modules::BindItem>()?;
 
     // create dummy object for all the type aliases we use in the stub file
     // TODO: maybe these should somehow be the actual types we use in the stub file
@@ -439,6 +459,9 @@ fn rust_circuit(py: Python<'_>, m: &PyModule) -> PyResult<()> {
         "NestAddsSpecSub",
         "NestAddsSpec",
         "GeneralFunctionSpec",
+        "Binder",
+        "UpdatedIterativeMatcher",
+        "UpdatedIterativeMatcherIn",
     ] {
         m.add(dummy_name, py.None())?;
     }
